/* Symbols in Navigation and ToC. */
:root, :host,
[data-md-color-scheme="default"] {
  --doc-symbol-parameter-fg-color: #df50af;
  --doc-symbol-attribute-fg-color: #953800;
  --doc-symbol-function-fg-color: #8250df;
  --doc-symbol-method-fg-color: #8250df;
  --doc-symbol-class-fg-color: #0550ae;
  --doc-symbol-module-fg-color: #5cad0f;

  --doc-symbol-parameter-bg-color: #df50af1a;
  --doc-symbol-attribute-bg-color: #9538001a;
  --doc-symbol-function-bg-color: #8250df1a;
  --doc-symbol-method-bg-color: #8250df1a;
  --doc-symbol-class-bg-color: #0550ae1a;
  --doc-symbol-module-bg-color: #5cad0f1a;
}

[data-md-color-scheme="dracula"] {
  --doc-symbol-parameter-fg-color: #ffa8cc;
  --doc-symbol-attribute-fg-color: #ffa657;
  --doc-symbol-function-fg-color: #d2a8ff;
  --doc-symbol-method-fg-color: #d2a8ff;
  --doc-symbol-class-fg-color: #79c0ff;
  --doc-symbol-module-fg-color: #baff79;

  --doc-symbol-parameter-bg-color: #ffa8cc1a;
  --doc-symbol-attribute-bg-color: #ffa6571a;
  --doc-symbol-function-bg-color: #d2a8ff1a;
  --doc-symbol-method-bg-color: #d2a8ff1a;
  --doc-symbol-class-bg-color: #79c0ff1a;
  --doc-symbol-module-bg-color: #baff791a;
}

code.doc-symbol-heading.doc-symbol-module {
    color: var(--doc-symbol-module-fg-color) !important;
    background-color: var(--doc-symbol-module-bg-color) !important;
}

code.doc-symbol-heading.doc-symbol-class {
    color: var(--doc-symbol-class-fg-color) !important;
    background-color: var(--doc-symbol-class-bg-color) !important;
}

code.doc-symbol-heading.doc-symbol-method {
    color: var(--doc-symbol-method-fg-color) !important;
    background-color: var(--doc-symbol-method-bg-color) !important;
}
