#!/usr/bin/env zsh

# Aliases for GIT shortcuts (https://laracasts.com/series/how-to-contribute-to-open-source/episodes/5)
alias wip='git add . && git commit -m '\''Work in progress'\'
alias nope='git reset --hard && git clean -f -d && git checkout HEAD'
