#!/usr/bin/env zsh

# ohmyzsh plugins from GitHub
setup-omz-plugins \
  zsh-users/zsh-autosuggestions \
  zsh-users/zsh-syntax-highlighting \
  zsh-users/zsh-completions \
  zsh-users/zsh-history-substring-search \
  matthiasha/zsh-uv-env
